"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var compose_1 = __importDefault(require("recompose/compose"));
var lifecycle_1 = __importDefault(require("recompose/lifecycle"));
var mapProps_1 = __importDefault(require("recompose/mapProps"));
var onlyUpdateForKeys_1 = __importDefault(require("recompose/onlyUpdateForKeys"));
var withState_1 = __importDefault(require("recompose/withState"));
/** Create an enhancer chain. */
function createEnhancerChain() {
    var enhancers = [];
    var enhancerChain = {
        startWith: function (e) { return enhancerChain.compose(e); },
        compose: function (e) {
            enhancers.push(e);
            return enhancerChain;
        },
        checkThis: function () { return enhancerChain; },
        forPropsOfType: function () { return enhancerChain; },
        enhance: function (c) { return compose_1.default.apply(void 0, enhancers)(c); },
        omitKeysFromInProps: function () { return enhancerChain; },
        omitKeysFromOutProps: function () { return enhancerChain; },
        keepKeysForInProps: function () { return enhancerChain; },
        keepKeysForOutProps: function () { return enhancerChain; },
        infuseWithProps: function (c) { return c; }
    };
    return enhancerChain;
}
exports.createEnhancerChain = createEnhancerChain;
/** Create a type-safe licycle HOC. */
function lifecycle(spec) {
    return lifecycle_1.default(spec);
}
exports.lifecycle = lifecycle;
/** Create a type-safe HOC to omit certain keys from props. */
function omitKeys() {
    var keys = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        keys[_i] = arguments[_i];
    }
    return mapProps_1.default(function (props) {
        var propCopy = __assign({}, props);
        keys.forEach(function (key) { return delete propCopy[key]; });
        return propCopy;
    });
}
exports.omitKeys = omitKeys;
/** Create a type-safe onlyUpdateForKeys. */
function onlyUpdateForKeys() {
    var keys = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        keys[_i] = arguments[_i];
    }
    return onlyUpdateForKeys_1.default(keys);
}
exports.onlyUpdateForKeys = onlyUpdateForKeys;
/** Create a type-safe withState HOC. */
function withState(stateName, stateUpdateFuncName, initialState) {
    return withState_1.default(stateName, stateUpdateFuncName, initialState);
}
exports.withState = withState;
